package com.os.ems.record.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.os.ems.record.mapper.EmsRecordSteamInstantMapper;
import com.os.ems.record.domain.EmsRecordSteamInstant;
import com.os.ems.record.service.IEmsRecordSteamInstantService;

/**
 * 蒸汽历史数据Service业务层处理
 *
 * @author Yinq
 * @date 2024-05-22
 */
@Service
public class EmsRecordSteamInstantServiceImpl implements IEmsRecordSteamInstantService {
    @Autowired
    private EmsRecordSteamInstantMapper emsRecordSteamInstantMapper;

    /**
     * 查询蒸汽历史数据
     *
     * @param objId 蒸汽历史数据主键
     * @return 蒸汽历史数据
     */
    @Override
    public EmsRecordSteamInstant selectEmsRecordSteamInstantByObjId(Long objId) {
        return emsRecordSteamInstantMapper.selectEmsRecordSteamInstantByObjId(objId);
    }

    /**
     * 查询蒸汽历史数据列表
     *
     * @param emsRecordSteamInstant 蒸汽历史数据
     * @return 蒸汽历史数据
     */
    @Override
    public List<EmsRecordSteamInstant> selectEmsRecordSteamInstantList(EmsRecordSteamInstant emsRecordSteamInstant) {
        return emsRecordSteamInstantMapper.selectEmsRecordSteamInstantList(emsRecordSteamInstant);
    }

    /**
     * 新增蒸汽历史数据
     *
     * @param emsRecordSteamInstant 蒸汽历史数据
     * @return 结果
     */
    @Override
    public int insertEmsRecordSteamInstant(EmsRecordSteamInstant emsRecordSteamInstant) {
        return emsRecordSteamInstantMapper.insertEmsRecordSteamInstant(emsRecordSteamInstant);
    }

    /**
     * 修改蒸汽历史数据
     *
     * @param emsRecordSteamInstant 蒸汽历史数据
     * @return 结果
     */
    @Override
    public int updateEmsRecordSteamInstant(EmsRecordSteamInstant emsRecordSteamInstant) {
        return emsRecordSteamInstantMapper.updateEmsRecordSteamInstant(emsRecordSteamInstant);
    }

    /**
     * 批量删除蒸汽历史数据
     *
     * @param objIds 需要删除的蒸汽历史数据主键
     * @return 结果
     */
    @Override
    public int deleteEmsRecordSteamInstantByObjIds(Long[] objIds) {
        return emsRecordSteamInstantMapper.deleteEmsRecordSteamInstantByObjIds(objIds);
    }

    /**
     * 删除蒸汽历史数据信息
     *
     * @param objId 蒸汽历史数据主键
     * @return 结果
     */
    @Override
    public int deleteEmsRecordSteamInstantByObjId(Long objId) {
        return emsRecordSteamInstantMapper.deleteEmsRecordSteamInstantByObjId(objId);
    }


    /**
     * 查询蒸汽实时最新一条数据列表
     *
     * @param emsRecordSteamInstant 蒸汽历史数据
     * @return 蒸汽实时数据
     */
    @Override
    public List<EmsRecordSteamInstant> selectEmsRecordLatestSteamInstantList(EmsRecordSteamInstant emsRecordSteamInstant){
        //获取监控编码
        String monitorCode = emsRecordSteamInstant.getMonitorCode();
        //初始化返回列表
        List<EmsRecordSteamInstant> emsRecordSteamInstants = new ArrayList<>();
        //判断监控编码是否为空
        if (monitorCode == null){
            //没有选择设备，monitorCode为空的时候，分组查询全部设备的最新一条记录
            List<EmsRecordSteamInstant> allEmsRecordSteamInstants = emsRecordSteamInstantMapper
                    .selectEmsRecordLatestSteamInstantList(emsRecordSteamInstant);
            //遍历查询结果
            for (EmsRecordSteamInstant recordSteamInstantallEmsRecordSteamInstants : allEmsRecordSteamInstants){
                //如果有设备名称就添加到返回集合中
                if (recordSteamInstantallEmsRecordSteamInstants.getMonitorName() != null){
                    emsRecordSteamInstants.add(recordSteamInstantallEmsRecordSteamInstants);
                }
            }
        }else {
            //选择了设备，monitorCode不为空的时候，根据monitorCode查询指定设备的最新一条记录
            emsRecordSteamInstants = emsRecordSteamInstantMapper
                    .selectEmsRecordLatestSteamInstantListByMonitorCode(emsRecordSteamInstant).subList(0, 1);
        }
        //返回查询结果
        return emsRecordSteamInstants;
    }


    /**
     * 根据时间段查询蒸汽历史数据列表
     *
     * @param emsRecordSteamInstant 蒸汽历史数据
     * @return 蒸汽历史数据集合
     */
    public List<EmsRecordSteamInstant> selectEmsRecordSteamInstantListByTime(EmsRecordSteamInstant emsRecordSteamInstant){
        return emsRecordSteamInstantMapper.selectEmsRecordSteamInstantListByTime(emsRecordSteamInstant);
    }

}
